#
# Copyright 2007 Fedora Unity
#
# Jonathan Steffan <jon a fedoraunity.org>
# Jeroen van Meeuwen <kanarip a fedoraunity.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import imgcreate
import logging
from ConfigParser import SafeConfigParser
from ConfigParser import RawConfigParser
import string
import subprocess
import time
import math
import re
import revisor
import rpmUtils.miscutils
import shutil
import fnmatch
import yum
import yum.misc
import yum.packageSack

from revisor.cfg import Defaults, Runtime
import revisor.cfg
import revisor.image
from revisor import misc
import revisor.progress
import revisor.kickstart as kickstart
import revisor.image

# Import constants
from revisor.constants import *

# Translation
from rhpl.translate import _

class RevisorBase:
    """RevisorBase holds common functions shared amongst our CLI and GUI mode"""
    def __init__(self, revisor):
        """Initializes the RevisorBase class with the options specified from the command line.
        Launches our plugin detection.
        Creates a logger instance
        Creates a configuration store
        Detects whether we are in CLI or GUI mode
        Sets the logger configuration
        Sets up the final configuration store
        Sets up yum as far as possible
        Sets up Revisor kickstart as far as possible"""

        # Get the options parser, it's valuable ;-)
        self.parser = revisor.parser

        # The options it has defined are valuable too
        self.cli_options = revisor.cli_options
        self.plugins = revisor.plugins
        self.plugins.base = self

        # At this point, 'self' isn't much yet, so:
        # first create a simple logger instance that won't do much,
        # then create a configuration store with that logger,
        # then start detecting the mode that we are in (GUI / CLI),
        # then let the logger know about the configuration store,
        # then /really/ set up the configuration store (now that it has a
        #     valid logger that knows about the configuration store),
        #
        # Create logger
        self.create_logger()

        # Create ConfigStore (it needs the logger to be created!)
        self.create_configstore()

        # Detect our mode (options or try/except)
        self.detect_mode()

        # Let the logger know about cfg (it needs a ConfigStore instance!)
        self.log.set_config(self.cfg)

        # Then really setup the ConfigStore (because that needs a logger!)
        self.cfg.setup_cfg()

        misc.check_selinux(log=self.log)

        self.cfg.setup_yum()
        self.cfg.setup_ks()

    def run(self):
        """Split into either running CLI, Server or GUI"""
        if self.cfg.cli_mode:
            import revisor.cli
            self.log.debug(_("Running Revisor in CLI mode..."), level=1)
            self.cli = revisor.cli.RevisorCLI(self)
            self.cli.run()

        elif hasattr(self.cfg,"server_mode"):
            if self.cfg.server_mode:
                self.log.debug(_("Running Revisor in RPC Server mode..."), level=1)
                self.server = self.plugins.modserver
                self.server.run(base=self)
            elif self.cfg.gui_mode:
                self.log.debug(_("Running Revisor in GUI mode..."), level=1)
                self.gui = self.plugins.modgui
                self.gui.run(base=self)

        elif self.cfg.gui_mode:
            self.log.debug(_("Running Revisor in GUI mode..."), level=1)
            self.gui = self.plugins.modgui
            self.gui.run(base=self)

    def create_logger(self):
        """Create a logger instance using cli_options.debuglevel"""
        if not self.cli_options.debuglevel == None:
            loglevel = logging.DEBUG
        else:
            loglevel = logging.INFO
            self.cli_options.debuglevel = 0

        # Initialize logger
        self.log = revisor.logger.Logger(loglevel=loglevel, debuglevel=self.cli_options.debuglevel)

    def create_configstore(self):
        """Initialize Configuration Store"""
        self.cfg = revisor.cfg.ConfigStore(self)

    def detect_mode(self):
        """Detect whether we run in CLI or in GUI mode. (GUI is default)

        Headless creates a RevisorBase object that does nothing on its own.
        Useful for spawning your army of Revisor daemons."""

        if self.cli_options.gui_mode:
            self.cfg._set_gui_mode()
        elif self.cli_options.cli_mode:
            self.cfg._set_cli_mode()
        elif hasattr(self.cli_options,"server_mode"):
            if not self.cli_options.server_mode:
                try:
                    import gtk
                    import gtk.glade
                    import gobject
                    import gtk.gdk as gdk
                    self.cfg._set_gui_mode()
                except:
                    self.cfg._set_cli_mode()
        else:
            try:
                import gtk
                import gtk.glade
                import gobject
                import gtk.gdk as gdk
                self.cfg._set_gui_mode()
            except:
                self.cfg._set_cli_mode()

    def show_help(self, keyword):
        """Some routine to trigger so that help (given a keyword) is being shown"""
        if self.cfg.gui_mode:
            self.mode.base_buttons_xml.get_widget("button_information").set_sensitive(False)
            pid = os.fork()
            if not pid:
                self.log.info(_("Opening up /usr/bin/htmlview %s%s") % (DOCS_BASEPATH, keyword))
                os.execv("/usr/bin/htmlview", ["/usr/bin/htmlview", "%s%s" % (DOCS_BASEPATH, keyword)])
            else:
#                self.mode.base_buttons_xml.get_widget("button_information").set_sensitive(True)
                self.log.error(_("Cannot fork process showing help, please use %s%s") % (DOCS_BASEPATH, keyword))
        else:
            self.log.warning(_("Cannot show Help in CLI mode, use %s%s") % (DOCS_BASEPATH, keyword))

    def setup_yum(self):
        """Setup the yum object"""

        # Let's make sure the cfg.setup_yum() is run
        self.cfg.setup_yum()

        self.cfg.yumobj.pbar = self.progress_bar(_("Loading Repositories"), callback=True)

        self.log.debug(_("Getting configuration from %s") % self.cfg.main, level=2)

        try:
            if hasattr(self.cfg.yumobj,"_getConfig"):
                self.cfg.yumobj._getConfig(fn=self.cfg.main, plugin_types=(yum.plugins.TYPE_CORE,))
            else:
                self.cfg.yumobj.doConfigSetup(fn=self.cfg.main, plugin_types=(yum.plugins.TYPE_CORE,))
                self.log.debug(_("Using deprecated YUM function: %s()") % "doConfigSetup", level=7)
        except:
            self.log.error(_("yum.YumBase.doConfigSetup failed, probably an invalid configuration file %s") % self.cfg.main, recoverable=False)

        if self.cfg.debuglevel > self.cfg.yumobj.conf.debuglevel:
            self.log.debug(_("Bumping YUMs debuglevel (%d) to our debuglevel (%d)") % (self.cfg.yumobj.conf.debuglevel,self.cfg.debuglevel), level=2)
            self.cfg.yumobj.conf.debuglevel=self.cfg.debuglevel
            self.log.debug(_("YUMs debuglevel now %d") % self.cfg.yumobj.conf.debuglevel, level=9)

        try:
            if not self.cfg.repo_override:
                self.cfg.reposSetup(callback=self.cfg.yumobj.pbar)
                repo_setup = True
            if self.cfg.repo_override:
                # Clear all the repos we have enabled from the model, and inject our override repo(s)
                for repo in self.cfg.yumobj.repos.repos.keys():
                    self.cfg.yumobj.repos.delete(str(repo))

                for repo in self.cfg.repo_override:
                    repoObj = yum.yumRepo.YumRepository(repo["name"])
                    repoObj.setAttribute('name', repo["name"])
                    repoObj.setAttribute('baseurl', repo["baseurl"])
                    repoObj.setAttribute('gpgcheck', repo["gpgcheck"])
                    self.cfg.yumobj.repos.add(repoObj)
                    self.cfg.yumobj.repos.enableRepo(repoObj.id)

                repo_setup=True

            #for repo in self.cfg.added_repos:
                #print "Adding custom repo %s to the yum repositories" % repo.id
                #repo.setAttribute('basecachedir', self.cfg.yumobj.conf.cachedir)
                #self.cfg.yumobj.repos.add(repo)
                #self.cfg.yumobj.repos.enableRepo(repo.id)

            #for repo in self.cfg.yumobj.repos.repos.keys():
                #print str(repo)

            #for repo in self.cfg.yumobj.repos.listEnabled():
                #print repo.name

        except yum.Errors.LockError:
            self.cfg.yumobj.pbar.destroy()
            self.log.error(_("Another application is running which is accessing software information."), recoverable=False)
            repo_setup = False

        except revisor.errors.RevisorDownloadError, e:
            self.cfg.yumobj.pbar.destroy()
            self.log.error(_("Fatal Error: Unable to retrieve software information.\n" + \
                            "\tThis could be caused by one of the following:\n" + \
                            "\t - not having a network connection available,\n" + \
                            "\t - Server refusing connections,\n" + \
                            "\t - Using a mirror that isn't fully synchronized,\n" + \
                            "\t - Misconfigured repositories."), recoverable=self.cfg.gui_mode)
            repo_setup = False

        if repo_setup:

            self.log.debug(_("Setting up a Transaction Set"), level=6)
            if hasattr(self.cfg.yumobj,"_getTs"):
                self.cfg.yumobj._getTs()
            else:
                self.log.debug(_("Using deprecated YUM function: %s()") % "doTsSetup", level=7)
                self.cfg.yumobj.doTsSetup()

            self.log.debug(_("Getting myself a piece of the RPMDB"), level=6)
            if hasattr(self.cfg.yumobj,"_getRpmDB"):
                self.cfg.yumobj._getRpmDB()
            elif hasattr(self.cfg.yumobj,"doSetupRPMDB"):
                self.log.debug(_("Using deprecated YUM function: %s()") % "doSetupRPMDB", level=7)
                self.cfg.yumobj.doSetupRPMDB()
            elif hasattr(self.cfg.yumobj,"doRpmDBSetup"):
                self.log.debug(_("Apparently we're running on an Enterprise Linux system (we can tell from the way yum sets up the RPM Database)"), level=7)
                self.cfg.yumobj.doRpmDBSetup()

            self.log.debug(_("Getting Repository Information"), level=6)
            if hasattr(self.cfg.yumobj,"_getRepos"):
                self.cfg.yumobj._getRepos()
            else:
                self.log.debug(_("Using deprecated YUM function: %s()") % "doRepoSetup", level=7)
                self.cfg.yumobj.doRepoSetup()

            self.log.debug(_("Arch list = %s") % self.cfg.arch_list, level=9)

            self.log.debug(_("Getting the Package Sacks"), level=6)
            try:
                if hasattr(self.cfg.yumobj,"_getSacks"):
                    self.cfg.yumobj._getSacks(archlist=self.cfg.arch_list)
                else:
                    self.log.debug(_("Using deprecated YUM function: %s()") % "doSackSetup", level=7)
                    self.cfg.yumobj.doSackSetup(archlist=self.cfg.arch_list)
            except:
                pass

            self.log.debug(_("All OK so far, %d packages in the Package Sack") % len(self.cfg.yumobj.pkgSack.simplePkgList()), level=9)

            self.cfg.yumobj.pbar.destroy()
            return True
        else:
            self.cfg.yumobj.pbar.destroy()
            return False

    def pkglist_from_ksdata(self, groupList=[], packageList=[], excludedList=[], ignore_list=[]):
        """Takes valid groupList, packageList and excludedList such as in kickstart package manifests,
        and selects packages from these lists. The way it does so depends on whether we are doing
        Installation Media Respins"""

        self.log.debug(_("Building a nice package list from ksdata, and adding it to the transaction"), level=6)

        # Actually, let's figure out what mode we're in and call something once we know which way to go

        if self.cfg.mode_respin and self.cfg.media_installation:
            self.log.debug(_("Kickstart mode: respin"))
            self.pkglist_from_ksdata_respin(groupList, packageList, excludedList)
        else:
            self.log.debug(_("Kickstart mode: normal"))
            self.pkglist_from_ksdata_normal(groupList, packageList, excludedList, ignore_list)

    def pkglist_from_ksdata_normal(self, groupList, packageList, excludedList, ignore_list=[]):
        """This function adds packages to the yum transaction set the regular way
        Pass it a list of names of packages that we should ignore the architecture off
        An example is libbeagle, kernel, pm-utils"""

        self.log.debug(_("What we're getting from pykickstart is: %d groups, %d packages and %d excluded packages. %d packages are being explicitly ignored.") % (len(groupList),len(packageList),len(excludedList),len(ignore_list)), level=9)
        total = float(len(groupList) * 3 +
                    len(packageList) * 2 +
                    len(excludedList))

        if total < 1.0:
            total = 1.0

        current = 0.0

        pbar = self.progress_bar(_("Select kickstart packages"))

        warnings = []

        # It seems anaconda always selects @core:
        # EL5: http://git.fedorahosted.org/git/?p=anaconda.git;a=blob;f=kickstart.py;h=3c188539fec95614efbeaa68764038ab1a2a6404;hb=0dcf8192c048324b718c3b0c2d212d1dfa584ac4
        # F8: http://git.fedorahosted.org/git/?p=anaconda.git;a=blob;f=kickstart.py;h=674ef804b01397d6e7fac3528d1f4f57882a9349;hb=b06e687cca2de7a138babfef5d0097a073dbdfc0#l956

        core_selected = False
        for ksgroup in groupList:
            if hasattr(ksgroup,"name"):
                group_name = ksgroup.name
            else:
                group_name = ksgroup

            if group_name == "core":
                core_selected = True

        if not core_selected and not ( self.cfg.kickstart_exact_nevra or self.cfg.kickstart_exact ):
            self.log.debug(_("Appending group @core"))
            groupList.append(self.cfg.ksobj._Group("core"))

        # See if 'base' is in the grouplist and whether we should add it
        base_selected = False
        for ksgroup in groupList:
            if hasattr(ksgroup,"name"):
                group_name = ksgroup.name
            else:
                group_name = ksgroup

            if group_name == "base":
                base_selected = True

        if not base_selected and self.cfg.ksobj._get("packages","addBase") and not ( self.cfg.kickstart_exact_nevra or self.cfg.kickstart_exact ):
            self.log.debug(_("Appending group @base"))
            groupList.append(self.cfg.ksobj._Group("base"))
##
## Add packages from Groups in ksdata
##
        for ksgroup in groupList:
            current += 3.0
            if hasattr(ksgroup,"name"):
                group_name = ksgroup.name
            else:
                group_name = ksgroup

            self.log.debug(_("Found group: %s") % group_name, level=8)
            # Get group object from comps
            # This might fail if not all repository data is available
            try:
                grp = self.cfg.yumobj.comps.return_group(group_name)
            except yum.Errors.RepoError, e:
                self.log.error(_("Repository metadata cannot be found: %s: %s") % ("yum.Errors.RepoError", e), recoverable=False)

            if not grp:
                warnings.append(_("Group not found: %s") % group_name)
                continue

            # Select the group
            txmbrs_used = self.cfg.yumobj.selectGroup(grp.name)

            # Add all required packages
            if hasattr(ksgroup,"include"):
                if ksgroup.include >= kickstart.constants.GROUP_REQUIRED:
                    self.log.debug(_("Selecting required packages for group %s") % grp.name, level=8)
                    for pkg in grp.mandatory_packages.keys():
                        if pkg in excludedList:
                            continue
                        self.log.debug(_("Including %s") % pkg, level=8)
                        # Get the package objects
                        try:
                            pkgs = self.cfg.yumobj.pkgSack._computeAggregateListResult("returnNewestByName", pkg)
                            if len(pkgs) > 1 and not pkg in ignore_list:
                                pkgs = self.cfg.yumobj.bestPackagesFromList(pkgs)

                            # Add to transaction
                            for po in pkgs:
                                self.cfg.yumobj.tsInfo.addInstall(po)
                                self.log.debug(_("From Groups: Adding %s-%s:%s-%s.%s to transaction") % (po.name, po.epoch, po.version, po.release, po.arch), level=9)
                        except yum.Errors.PackageSackError, e:
                            if self.cfg.ksobj._get("packages","handleMissing") != kickstart.constants.KS_MISSING_IGNORE and not group_name == "core":
                                warnings.append(e.value)

                # Add all default packages
                if ksgroup.include >= kickstart.constants.GROUP_DEFAULT:
                    self.log.debug(_("Selecting default packages for group %s") % grp.name, level=8)
                    for pkg in grp.default_packages.keys():
                        if pkg in excludedList:
                            continue
                        self.log.debug(_("Including %s") % pkg, level=9)
                        # Get the package objects
                        try:
                            pkgs = self.cfg.yumobj.pkgSack.returnNewestByName(pkg)
                            if len(pkgs) > 1 and not pkg in ignore_list:
                                pkgs = self.cfg.yumobj.bestPackagesFromList(pkgs)
                            # Add to transaction
                            for po in pkgs:
                                self.cfg.yumobj.tsInfo.addInstall(po)
                                self.log.debug(_("From Groups: Adding %s-%s:%s-%s.%s to transaction") % (po.name, po.epoch, po.version, po.release, po.arch), level=9)
                        except yum.Errors.PackageSackError, e:
                            if self.cfg.ksobj._get("packages","handleMissing") != kickstart.constants.KS_MISSING_IGNORE and not group_name == "core":
                                warnings.append(e.value)

                # Add all optional packages
                if ksgroup.include >= kickstart.constants.GROUP_ALL:
                    self.log.debug(_("Selecting optional packages for group %s") % grp.name, level=7)
                    # All optional packages included, get the package names
                    for pkg in grp.optional_packages.keys():
                        if pkg in excludedList:
                            continue
                        self.log.debug(_("Including %s") % pkg, level=8)
                        # Get the package objects
                        try:
                            pkgs = self.cfg.yumobj.pkgSack.returnNewestByName(pkg)
                            if len(pkgs) > 1 and not pkg in ignore_list:
                                pkgs = self.cfg.yumobj.bestPackagesFromList(pkgs)
                            # Add to transaction
                            for po in pkgs:
                                self.cfg.yumobj.tsInfo.addInstall(po)
                                self.log.debug(_("From Groups --optional: Adding %s-%s:%s-%s.%s to transaction") % (po.name, po.epoch, po.version, po.release, po.arch), level=9)
                        except yum.Errors.PackageSackError, e:
                            if self.cfg.ksobj._get("packages","handleMissing") != kickstart.constants.KS_MISSING_IGNORE and not group_name == "core":
                                warnings.append(e.value)
                pbar.set_fraction(current/total)
            else:
                self.log.debug(_("No include parameter for group %s, using defaults") % group_name, level=8)
                for pkg in grp.default_packages.keys():
                    if pkg in excludedList:
                        continue
                    self.log.debug(_("Including %s") % pkg, level=8)
                    # Get the package objects
                    try:
                        pkgs = self.cfg.yumobj.pkgSack.returnNewestByName(pkg)
                        if len(pkgs) > 1 and not pkg in ignore_list:
                            pkgs = self.cfg.yumobj.bestPackagesFromList(pkgs)
                        # Add to transaction
                        for po in pkgs:
                            self.cfg.yumobj.tsInfo.addInstall(po)
                            self.log.debug(_("From Groups --optional: Adding %s-%s:%s-%s.%s to transaction") % (po.name, po.epoch, po.version, po.release, po.arch), level=9)
                    except yum.Errors.PackageSackError, e:
                        if self.cfg.ksobj._get("packages","handleMissing") != kickstart.constants.KS_MISSING_IGNORE and not group_name == "core":
                            warnings.append(e.value)

##
## Pull the conditional packages in
##
            for condreq, cond in grp.conditional_packages.iteritems():
                self.log.debug(_("Testing condition: %s / %s") % (condreq, cond), level=9)
                pkgs = self.cfg.yumobj.pkgSack.searchNevra(name=condreq)
                if len(pkgs) > 1:
                    pkgs = self.cfg.yumobj.bestPackagesFromList(pkgs)
                if self.cfg.yumobj.tsInfo.conditionals.has_key(cond):
                    self.cfg.yumobj.tsInfo.conditionals[cond].extend(pkgs)
                else:
                    self.cfg.yumobj.tsInfo.conditionals[cond] = pkgs
##
## Add packages in ksdata to transaction
##
        for pkg in packageList:
            search = False
            current += 2.0
            self.log.debug(_("From package list, including: %s") % pkg, level=8)

            if pkg in excludedList:
                self.log.debug(_("Package %s is in excludeList, continuing") % pkg, level=9)
                continue

            try:
                pkgs = self.cfg.yumobj.pkgSack.returnNewestByName(pkg)
                if len(pkgs) > 1 and not pkg in ignore_list:
                    pkgs = self.cfg.yumobj.bestPackagesFromList(pkgs)
                elif len(pkgs) == 0:
                    self.log.debug(_("No packages found!"), level=9)
                    search = True

                for po in pkgs:
                    self.cfg.yumobj.tsInfo.addInstall(po)
                    self.log.debug(_("From Packages: Adding %s-%s:%s-%s.%s to transaction") % (po.name, po.epoch, po.version, po.release, po.arch), level=9)

            except yum.Errors.PackageSackError, e:
                search = True

            # Let's see if there's some regexp, and if there is, let's search
            if search:
                self.log.debug(_("Could not find package '%s', searching...") % pkg, level=9)
                if "*" in pkg:
                    try:
                        pkglist = self.cfg.yumobj.pkgSack.simplePkgList()
                        matches = []
                        # anything we couldn't find a match for
                        # could mean it's not there, could mean it's a wildcard
                        if re.match('.*[\*,\[,\],\{,\},\?].*', pkg):
                            restring = fnmatch.translate(pkg)
                            regex = re.compile(restring, flags=re.I) # case insensitive
                            for item in pkglist:
                                if regex.match(item[0]):
                                    matches.append(item[0])
                                    self.log.debug(_("Found packages matching '%s': %s") % (pkg,item[0]), level=9)
                        for pkg in matches:
                            pkgs = self.cfg.yumobj.pkgSack.returnNewestByName(pkg)
                            if len(pkgs) > 1:
                                pkgs = self.cfg.yumobj.bestPackagesFromList(pkgs)

                            for po in pkgs:
                                if not po.name in ignore_list:
                                    self.cfg.yumobj.tsInfo.addInstall(po)
                                    self.log.debug(_("From Packages: Adding %s-%s:%s-%s.%s to transaction") % (po.name, po.epoch, po.version, po.release, po.arch), level=9)
                                else:
                                    self.log.debug(_("From Packages: Not adding %s now because it is in the ignore list") % po.name, level=9)

                    except yum.Errors.PackageSackError, e:
                        if self.cfg.ksobj._get("packages","handleMissing") != kickstart.constants.KS_MISSING_IGNORE:
                            warnings.append(e.value)
                        # Here's where we try and see if this is a rpm -qa list
                elif re.match('.*-.*-.*\..*', pkg):
                    (name, ver, relarch) = pkg.rsplit('-', 2)
                    (rel, arch) = relarch.rsplit('.', 1)
                    pkgs = self.cfg.yumobj.pkgSack.searchNevra(name=name, ver=ver, rel=rel, arch=arch)
                    if len(pkgs) > 1:
                        warnings.append(_("More then one package found for %s-%s-%s.%s - going to add them all to the transaction") % (name,ver,rel,arch))
                    elif len(pkgs) < 1:
                        warnings.append(_("Could not find package %s-%s-%s.%s") % (name,ver,rel,arch))
                    for po in pkgs:
                        self.cfg.yumobj.tsInfo.addInstall(po)
                        self.log.debug(_("From Packages (exact string %s-%s-%s.%s), selecting %s-%s-%s.%s") % (name,ver,rel,arch,po.name,po.version,po.release,po.arch), level=9)

                else:
                    try:
                        if self.cfg.ksobj._get("packages","handleMissing") != kickstart.constants.KS_MISSING_IGNORE:
                            warnings.append(e.value)
                    except UnboundLocalError:
                        pass

            pbar.set_fraction(current/total)

##
## Exclude packages from ksdata
##
        for pkg in excludedList:
            current += 1.0
            try:
                pkgs = self.cfg.yumobj.pkgSack.returnNewestByName(pkg)
                for po in pkgs:
                    self.log.debug(_("From Excludes: Removing %s-%s:%s-%s.%s from transaction") % (po.name, po.epoch, po.version, po.release, po.arch), level=8)
                    self.cfg.yumobj.tsInfo.remove(po.pkgtup)
            except yum.Errors.PackageSackError, e:
                self.log.debug(_("Apparently trying to exclude a package that is not available in the repositories loaded, or hasn't been added to the transaction: %s") % e.value, level=4)
            except:
                pass
            pbar.set_fraction(current/total)

        if len(warnings) > 0:
            self.log.warning(_("\nThe following errors occured when selecting groups and packages from kickstart:\n\n- %s\n\nYou can continue with these minor errors but obviously the results may not be what you expected.") % '\n- '.join(warnings))

#        self.cfg.yumobj.tsInfo.makelists()

        self.cfg.ts_length_pre_depsolve = len(self.cfg.yumobj.tsInfo.getMembers())

        self.log.debug(_("This is what was selected to be installed:"), level=7)
        installpkgs = []
        for po in self.cfg.yumobj.tsInfo.installed:
            installpkgs.append(po.pkgtup)
            self.log.debug("--> %s" % str(po.pkgtup), level=7)

        pbar.destroy()

    def pkglist_from_ksdata_respin(self, groupList, packageList, excludedList):
        """Takes valid kshandler.packages.groupList, kshandler.packages.packageList
        and kshandler.packages.excludedList, and builds a nice and simple list that
        we can add our required packages to, and then start depsolving"""

        self.log.debug(_("What we're getting from pykickstart is: %d groups, %d packages and %d excluded packages.") % (len(groupList),len(packageList),len(excludedList)), level=9)

        pkglist = []
        matchdict = {}

        total = 1.0
        current = 0.0

        pbar = self.progress_bar(_("Select kickstart packages"))

        # It seems anaconda always selects @core:
        # EL5: http://git.fedorahosted.org/git/?p=anaconda.git;a=blob;f=kickstart.py;h=3c188539fec95614efbeaa68764038ab1a2a6404;hb=0dcf8192c048324b718c3b0c2d212d1dfa584ac4
        # F8: http://git.fedorahosted.org/git/?p=anaconda.git;a=blob;f=kickstart.py;h=674ef804b01397d6e7fac3528d1f4f57882a9349;hb=b06e687cca2de7a138babfef5d0097a073dbdfc0#l956

        core_selected = False
        for ksgroup in groupList:
            if hasattr(ksgroup,"name"):
                group_name = ksgroup.name
            else:
                group_name = ksgroup

            if group_name == "core":
                core_selected = True

        if not core_selected and not ( self.cfg.kickstart_exact_nevra or self.cfg.kickstart_exact ):
            self.log.debug(_("Appending group @core"))
            groupList.append(self.cfg.ksobj._Group("core"))

        # See if 'base' is in the grouplist and whether we should add it
        base_selected = False
        for ksgroup in groupList:
            if hasattr(ksgroup,"name"):
                group_name = ksgroup.name
            else:
                group_name = ksgroup

            if group_name == "base":
                base_selected = True

        if not base_selected and self.cfg.ksobj._get("packages","addBase") and not ( self.cfg.kickstart_exact_nevra or self.cfg.kickstart_exact ):
            self.log.debug(_("Appending group @base"))
            groupList.append(self.cfg.ksobj._Group("base"))

##
## Add packages from Groups in ksdata
##
        for ksgroup in groupList:
            if not self.cfg.yumobj.comps.has_group(ksgroup.name):
                self.log.warning(_("No such group %s") % ksgroup.name )
                continue
            else:
                self.log.debug(_("Found group: %s") % ksgroup.name, level=8)

            # Get group object from comps
            grp = self.cfg.yumobj.comps.return_group(ksgroup.name)
            txmbrs_used = self.cfg.yumobj.selectGroup(grp.name)

            if ksgroup.include >= kickstart.constants.GROUP_REQUIRED:
                pkglist.extend(grp.mandatory_packages.keys())
            if ksgroup.include >= kickstart.constants.GROUP_DEFAULT:
                pkglist.extend(grp.default_packages.keys())
            if ksgroup.include >= kickstart.constants.GROUP_ALL:
                pkglist.extend(grp.optional_packages.keys())

##
## Pull the conditional packages in
##
            for condreq, cond in grp.conditional_packages.iteritems():
                self.log.debug(_("Testing condition: %s / %s") % (condreq, cond), level=8)
                pkgs = self.cfg.yumobj.pkgSack.searchNevra(name=condreq)
                if len(pkgs) > 1:
                    pkgs = self.cfg.yumobj.bestPackagesFromList(pkgs)
                if self.cfg.yumobj.tsInfo.conditionals.has_key(cond):
                    self.cfg.yumobj.tsInfo.conditionals[cond].extend(pkgs)
                else:
                    self.cfg.yumobj.tsInfo.conditionals[cond] = pkgs

##
## Add packages in ksdata to transaction
##
        for pkg in packageList:
            pkglist.append(pkg)

##
## Exclude packages from ksdata
##
        for pkg in excludedList:
            if pkg in pkglist:
                self.cfg.yumobj.conf.exclude.extend(pkg)

##
## Add pkglist to install
##
        # Make the search list unique
        pkglist = yum.misc.unique(pkglist)

        total = float(len(pkglist))

        # Search repos for things in our searchlist, supports globs
        (exactmatched, matched, unmatched) = yum.packages.parsePackages(self.cfg.yumobj.pkgSack.returnPackages(), pkglist, casematch=1)
        matches = exactmatched + matched

        # Populate a dict of package objects to their names
        for match in matches:
            matchdict[match.name] = match

        # Get the newest results from the search
        mysack = yum.packageSack.ListPackageSack(matches)
        for match in mysack.returnNewestByNameArch():
            current += 1.0
            self.cfg.yumobj.tsInfo.addInstall(match)
            pbar.set_fraction(current/total)
            self.log.debug(_("Adding %s-%s:%s-%s.%s") % (match.name, match.epoch, match.version, match.release, match.arch), level=9)

        self.cfg.ts_length_pre_depsolve = len(self.cfg.yumobj.tsInfo.getMembers())

        pbar.destroy()

    def progress_bar(self, title="", parent=None, xml=None, callback=False):
        """This function should be used to determine the type of progress bar we need.
        There's three types:
            - GUI Dialog Progress Bar (separate dialog, window pops up)
            - GUI Nested Progress Bar (no separate dialog, progress bar is in gui.frame_xml)
            - CLI Progress Bar

        This function also determines whether we should have a Callback Progress Bar"""

        self.log.debug(_("Initting progress bar for ") + title, level=9)

        if self.cfg.gui_mode:
            try:
                if hasattr(self.gui,"BuildMedia"):
                    self.gui.BuildMedia.show_task_list()
            except:
                self.log.debug(_("Apparently we have not yet entered the Build Media stage"), level=9)

            if callback:
                if not self.gui.frame_xml.get_widget("part_progress") == None:
                    return revisor.progress.ProgressCallbackGUI(title=title, parent=self.gui, xml=self.gui.frame_xml)
                elif not self.gui.main_window == None:
                    return revisor.progress.ProgressCallbackGUI(title=title, parent=self.gui, xml=self.gui.frame_xml)
                else:
                    return revisor.progress.ProgressCallbackGUI(title=title, parent=parent, xml=xml)
            else:
                if not self.gui.frame_xml.get_widget("part_progress") == None:
                    return revisor.progress.ProgressGUI(title=title, parent=self.gui, xml=self.gui.frame_xml)
                elif not self.gui.main_window == None:
                    return revisor.progress.ProgressGUI(title=title, parent=self.gui, xml=xml)
                else:
                    return revisor.progress.ProgressGUI(title=title, parent=parent, xml=xml)
        else:
            if callback:
                return revisor.progress.ProgressCallbackCLI(title=title)
            else:
                return revisor.progress.ProgressCLI(title=title)

#    def set_mode(self, mode):
#        self.mode = mode

    def get_package_deps(self, po, pbar):
        """Add the dependencies for a given package to the
           transaction info"""

        self.log.debug(_("Checking dependencies for %s.%s") % (po.name, po.arch), level=8)

        pbar.cur_task += 1.0
        reqs = po.requires
        provs = po.provides

        for req in reqs:
            if self.resolved_deps.has_key(req):
                pbar.cur_task += 1.0
                pbar.set_fraction(pbar.cur_task/pbar.num_tasks)
                continue

            (r,f,v) = req
            if r.startswith('rpmlib(') or r.startswith('config('):
                pbar.cur_task += 1.0
                pbar.set_fraction(pbar.cur_task/pbar.num_tasks)
                continue

            deps = self.cfg.yumobj.whatProvides(r, f, v).returnPackages()

            if not deps:
#                pbar.cur_task += 1.0
                self.log.warning(_("Unresolvable dependency %s %s %s in %s.%s") % (r, f, v, po.name, po.arch))
                continue

            depsack = yum.packageSack.ListPackageSack(deps)

            for dep in depsack.returnNewestByNameArch():
                self.cfg.yumobj.tsInfo.addInstall(dep)
                self.log.debug(_("Added %s-%s:%s-%s.%s for %s-%s:%s-%s.%s (requiring %s %s %s)") % (dep.name, dep.epoch, dep.version, dep.release, dep.arch, po.name, po.epoch, po.version, po.release, po.arch, r, f, v), level=9)

    def check_dependencies_conflicts(self):
        self.log.debug(_("Checking dependencies - allowing conflicts within the package set"), level=5)
        pbar = self.progress_bar(_("Resolving Dependencies"), callback=True)
        pbar.num_tasks = float(len(self.cfg.yumobj.tsInfo.getMembers()))

        # Hook in modrebrand to replace fedora-logos
        self.cfg.plugins.exec_hook("pre_resolve_dependencies")

        revisor.misc.resolve_dependencies_ugly(self.cfg.yumobj, logger=self.log, pbar=pbar)

        # Hook in modrebrand to replace fedora-logos
        self.cfg.plugins.exec_hook("post_resolve_dependencies")

        pbar.destroy()

    def check_dependencies_no_conflicts(self):
        """Check the dependencies"""
        self.log.debug(_("Checking dependencies - not allowing any conflicts within the package set"), level=5)
        pbar = self.progress_bar(_("Resolving Dependencies"), callback=True)

        # Hook in modrebrand to replace fedora-logos
        self.cfg.plugins.exec_hook("pre_resolve_dependencies")

        dsCB = revisor.progress.dscb(pbar, self.cfg.yumobj, self.cfg)
        self.cfg.yumobj.dsCallback = dsCB

        try:
            (result, msgs) = self.cfg.yumobj.buildTransaction()
        except yum.Errors.RepoError, errmsg:
            self.cfg.yumobj.dsCallback = None
            pbar.destroy()

            self.log.error(_("""Errors where encountered while downloading package headers:\n\n%s""" % (errmsg)))

        self.cfg.yumobj.dsCallback = None
        pbar.destroy()

        # FIXME:
        # - Dependency resolving may fail yet continue in case of composing installation media
        #   - which is not supposed to go to cobbler
        #   - which does not include the kickstart

        self.cfg.recoverable=True

        if result == 1:
            # We at least now that live media is going to fail (transaction cannot be executed)
            if self.cfg.media_live:
                self.cfg.recoverable=self.cfg.gui_mode
            elif self.cfg.kickstart_include:
                self.cfg.recoverable=self.cfg.gui_mode
            else:
                self.cfg.plugins.exec_hook("dependency_resolve_fail")

            self.log.error(_("""Unable to resolve dependencies for some packages selected:\n\n%s""" % (string.join(msgs, "\n"))), recoverable=self.cfg.recoverable)

        # End of dependency resolving

        self.cfg.yumobj.tsInfo.makelists()
        if not len(self.cfg.yumobj.tsInfo.getMembers()) == self.cfg.ts_length_pre_depsolve and self.cfg.kickstart_exact_nevra:
            self.log.warning("%d packages now vs. %d from the kickstart package manifest" % (len(self.cfg.yumobj.tsInfo.getMembers()), self.cfg.ts_length_pre_depsolve))
            self.log.error(_("The package set after dependency resolving does not match the packages selected in the kickstart manifest"), recoverable=False)
            sys.exit(0)

    def check_dependencies(self, style=None):
        if self.cfg.everything_spin: return

        if not style:
            if self.cfg.dependency_resolve_allow_conflicts:
                self.check_dependencies_conflicts()
            else:
                self.check_dependencies_no_conflicts()
        else:
            style()

        self.populate_stats()

    def enable_source_repositories(self):
        """For each enabled repository, searches the "%s-source" % repo.id equivalent and
        enables it, gets the repository metadata and gets the package sack."""

        # Reset the existing sacks
        self.cfg.yumobj._pkgSack = None

        archlist = self.cfg.arch_list + ['src']
        for repo in self.cfg.yumobj.repos.listEnabled():
            srcrepo = '%s-source' % repo.id
            if len(self.cfg.yumobj.repos.findRepos(srcrepo)) < 1:
                self.log.error(_("No such repository: %s") % srcrepo)
            for r in self.cfg.yumobj.repos.findRepos(srcrepo):
                self.log.debug(_("Enabling %s repository") % r.id, level=5)
                r.enable()
                # Setup the repo
                self.cfg.yumobj._getRepos(thisrepo=r.id,doSetup=True)
                # Setup pkgSack with 'src' in the archlist
                try:
                    self.cfg.yumobj._getSacks(archlist=archlist,thisrepo=r.id)
                except yum.Errors.RepoError, e:
                    self.log.error(e)

    def disable_source_repositories(self):
        # Reset the existing sacks
        self.cfg.yumobj._pkgSack = None

        for repo in self.cfg.yumobj.repos.listEnabled():
            if repo.id.endswith("-source"):
                self.log.debug(_("Disabling %s repository") % repo.id, level=5)
                repo.disable()
            else:
                # Setup the repo
                self.cfg.yumobj._getRepos(thisrepo=repo.id,doSetup=True)
                # Setup pkgSack with 'src' in the archlist
                try:
                    self.cfg.yumobj._getSacks(archlist=self.cfg.arch_list,thisrepo=repo.id)
                except yum.Errors.RepoError, e:
                    self.log.error(e)

    def download_source_packages(self):
        """Download source packages using self.polist"""
        self.enable_source_repositories()

        for po in self.polist:
            srpm = po.sourcerpm.split('.src.rpm')[0]
            if not srpm in self.srpmlist:
                self.srpmlist.append(srpm)

        for srpm in self.srpmlist:
            (sname, sver, srel) = srpm.rsplit('-', 2)
            try:
                srpmpos = self.cfg.yumobj.pkgSack.searchNevra(name=sname, ver=sver, rel=srel)
                for srpmpo in srpmpos:
                    if "source" in srpmpo.repoid:
                        if not srpmpo in self.srpmpolist:
                            self.srpmpolist.append(srpmpo)
            except IndexError:
                self.log.error(_("Error: Cannot find a source rpm for %s") % srpm)

        pbar = self.progress_bar(_("Downloading Source Packages"))
        total = len(self.srpmpolist)
        current = 0.0
        pbar.set_fraction(0.0)
        self.prev_download = ""
        self.attempt = 1
        self.current = 1

        for pkg in self.srpmpolist:
            n,a,e,v,r = pkg.pkgtup
            packages =  self.cfg.yumobj.pkgSack.searchNevra(n,e,v,r,a)
            for download in packages:
                repo = self.cfg.yumobj.repos.getRepo(download.repoid)
                if hasattr(download,"returnSimple"):
                    remote = download.returnSimple('relativepath')
                else:
                    remote = download.relativepath
                if os.path.exists(download.localPkg()) and os.path.getsize(download.localPkg()) == int(download.returnSimple('packagesize')):
                    continue
                path = repo.getPackage(download)
                if not os.path.exists(download.localPkg()) and os.path.exists(path):
                    try:
                        os.link(path,download.localPkg())
                    except:
                        if self.cfg.copy_local:
                            shutil.copy2(path,download.localPkg())
                        else:
                            os.symlink(path,download.localPkg())

                txt = _("Downloading %s") % remote
                if self.prev_download == "":
                    self.prev_download = download
                elif self.prev_download == download:
                    self.attempt += 1
                    self.current -= 1
                else:
                    self.attempt = 1
                    self.prev_download = download

                if self.attempt > 1:
                    txt = "#%d: %s" % (self.attempt,txt)
                self.log.debug(txt, level=1)
            current += 1
            pbar.set_fraction(current/total)

        pbar.destroy()

        self.disable_source_repositories()

    def download_packages(self):

        self.cfg.plugins.exec_hook("pre_download_packages")

        dlpkgs = map(lambda x: x.po, filter(lambda txmbr:
                                            txmbr.ts_state in ("i", "u"),
                                            self.cfg.yumobj.tsInfo.getMembers()))

        self.polist = dlpkgs

        pbar = self.progress_bar(_("Downloading Packages"))

        dlCb = revisor.progress.dlcb(pbar, dlpkgs, log=self.log, cfg=self.cfg)
        self.cfg.yumobj.repos.setProgressBar(dlCb)
        try:
            probs = self.cfg.yumobj.downloadPkgs(dlpkgs, dlCb)
        except yum.Errors.RepoError, errmsg:
            self.log.error(errmsg)
        except IndexError:
            self.log.error(_("Unable to find a suitable mirror."))

        self.cfg.yumobj.repos.setProgressBar(None)

        if len(probs.keys()) > 0:
            errstr = []
            for key in probs.keys():
                errors = yum.misc.unique(probs[key])
                for error in errors:
                    errstr.append("%s: %s" %(key, error))

            details_str = string.join(errstr, "\n")
            self.log.error(_("Errors were encountered while downloading packages: %s") % details_str, recoverable=self.cfg.gui_mode)

        pbar.destroy()

        self.cfg.plugins.exec_hook("post_download_packages")

        if self.cfg.getsource:
            self.download_source_packages()

    def pkglist_required(self, mode='installation'):

        required_pkgs = revisor.misc.resolve_pkgs(self.cfg.yumobj, self.cfg.get_package_list(mode,['require']), log=self.log)
        suggested_pkgs = revisor.misc.resolve_pkgs(self.cfg.yumobj, self.cfg.get_package_list(mode,['suggest']), log=self.log)
        allarch_pkgs = revisor.misc.resolve_pkgs(self.cfg.yumobj, self.cfg.get_package_list(mode,['allarch']), log=self.log)
        onearch_pkgs = revisor.misc.resolve_pkgs(self.cfg.yumobj, self.cfg.get_package_list(mode,['onearch']), log=self.log)

        packages_to_add = required_pkgs + suggested_pkgs + allarch_pkgs + onearch_pkgs
        packages_to_skip = []

        # These versions do not require you include anything anymore, as they compose
        # against the external repositories used to compose the tree, rather then
        # the tree itself

        if self.cfg.version_from in [ "F9", "DEVEL" ] and mode == "installation":
            packages_to_add = []
            required_pkgs = ['kernel', 'kernel-xen']
            suggested_pkgs = []

        # Make sure that if we are not in respin mode, we add the flexibility of overriding what we select
        # as required, by skipping adding extra or other packages with out list if they are already in the transaction
        # (from kickstart, for those who were wandering)
        for pkg_tuple in self.cfg.yumobj.tsInfo.pkgdict.keys():
            # Make sure we don't delete any user selected options with "best" package magic
            for package in packages_to_add:
                if str(pkg_tuple[0]) == str(package) and not package in allarch_pkgs:
                    self.log.debug(_("Overriding auto package selection with user package selection for %s..." % package), level=4)
                    packages_to_skip.append(package)

        for pkg in required_pkgs:

            try:
                pkgs = self.cfg.yumobj.pkgSack.returnNewestByName(pkg)

                if not pkg in packages_to_skip:
                    if not pkg in allarch_pkgs or pkg in onearch_pkgs:
                        pkgs = self.cfg.yumobj.bestPackagesFromList(pkgs)
                for po in pkgs:
                    self.cfg.yumobj.tsInfo.addInstall(po)
                    self.log.debug(_("Adding required package %s-%s:%s-%s.%s") % (po.name, po.epoch, po.version, po.release, po.arch), level=4)
            except yum.Errors.PackageSackError, e:
                # This list has already been resolved
                self.log.error(_("%s. This is a required package.") % e.value, recoverable=self.cfg.gui_mode)
                return False

        for pkg in suggested_pkgs:
            # If available, add them
            try:
                pkgs = self.cfg.yumobj.pkgSack.returnNewestByName(pkg)
                if not pkg in packages_to_skip:
                    if not pkg in allarch_pkgs or pkg in onearch_pkgs:
                        pkgs = self.cfg.yumobj.bestPackagesFromList(pkgs)
                for po in pkgs:
                    self.cfg.yumobj.tsInfo.addInstall(po)
                    self.log.debug(_("Adding suggested package %s-%s:%s-%s.%s") % (po.name, po.epoch, po.version, po.release, po.arch), level=4)
            except:
                pass

        self.added_pkgs = []

        for package in packages_to_add:
            if not package in packages_to_skip:
                self.added_pkgs.append(package)

        self.log.debug(_("Packages that do not need to be on the media: %s") % str(self.added_pkgs), level=9)
        return True

    def lift_off(self):
        """Do the actual thing. Dance the salsa."""
        self.srpmlist = []
        self.srpmpolist = []
        self.polist = []

        groupList = self.cfg.ksobj._get("packages","groupList")
        packageList = self.cfg.ksobj._get("packages","packageList")
        excludedList = self.cfg.ksobj._get("packages","excludedList")

        if self.cfg.media_installation:
            ignore_list = self.cfg.get_package_list(['installation'],['allarch'],['all',self.cfg.architecture],['all',self.cfg.version_from])

            if (self.cfg.kickstart_manifest and not self.cfg.i_did_all_this) or not self.cfg.gui_mode:
                self.pkglist_from_ksdata(groupList=groupList, packageList=packageList, excludedList=excludedList, ignore_list=ignore_list)

            if self.pkglist_required():
                self.check_dependencies()
                self.report_sizes()
                self.download_packages()
                self.buildInstallationMedia()
            else:
                self.log.error(_("Did not succeed in adding in all required packages"), recoverable=False)

#            # If we've done installation media, we want to remove all packages we added
#            # as required packages and that were not in the original list
#            for pkg in self.cfg.yumobj.tsInfo.pkgdict.keys():
#                if not pkg in self.cfg.pkglist_selected_tups:
#                    self.log.debug(_("Removing %s from tsInfo") % str(pkg))
#                    self.cfg.yumobj.tsInfo.remove(pkg)
#                else:
#                    self.log.debug(_("Not removing %s from tsInfo") % str(pkg))

        if self.cfg.media_live:
            ignore_list = self.cfg.get_package_list(['live'],['allarch'],['all',self.cfg.architecture],['all',self.cfg.version_from])
            if self.cfg.gui_mode:
                if not self.cfg.i_did_all_this:
                    self.pkglist_from_ksdata(groupList=groupList, packageList=packageList, excludedList=excludedList, ignore_list=ignore_list)
            else:
                self.pkglist_from_ksdata(groupList=groupList, packageList=packageList, excludedList=excludedList, ignore_list=ignore_list)
#            self.check_dependencies(style=self.check_dependencies_no_conflicts)

            if self.pkglist_required(mode='live'):
                self.check_dependencies(style=self.check_dependencies_no_conflicts)
                self.report_sizes()
                self.download_packages()
                self.buildLiveMedia()
            else:
                self.log.error(_("Did not succeed in adding in all required packages"))

        if self.cfg.gui_mode:
            # Show Finshed Screen
            self.gui.displayFinished()

    def populate_stats(self):
        """ Populate the stats displayed on the ready screen, based on our yum transaction. """
        pbar = self.progress_bar(_("Populating statistics"))
        pkgs = self.cfg.yumobj.tsInfo.getMembers()
        self.cfg.payload_packages = len(pkgs)
        pkgsize = 0
        installsize = 0
        current = 0.0
        total = float(len(pkgs))
        for pkg in pkgs:
            current += 1.0
            try:
                pkgsize += pkg.po.packagesize
            except KeyError, e:
                self.log.debug(_("Package %s-%s:%s-%s.%s does not seem to have a package size header") % (pkg.po.name, pkg.po.epoch, pkg.po.version, pkg.po.release, pkg.po.arch))
            except AttributeError, e:
#                self.log.debug(_("No attribute packagesize for package %s") % pkg.po.name)
                pkgsize += int(pkg.po.returnSimple('packagesize'))

            try:
                installsize += pkg.po.installedsize
            except KeyError, e:
                self.log.debug(_("Package %s-%s:%s-%s.%s does not seem to have a package size header") % (pkg.po.name, pkg.po.epoch, pkg.po.version, pkg.po.release, pkg.po.arch))
            except AttributeError, e:
#                self.log.debug(_("No attribute installedsize for package %s") % pkg.po.name)
                installsize += int(pkg.po.returnSimple('installedsize'))

            pbar.set_fraction(total/current)

        self.cfg.payload_installmedia = pkgsize / 1024 / 1024
        pkgsize = str(self.cfg.payload_installmedia) + " MB"
        self.cfg.payload_livemedia = installsize / 1024 / 1024
        installsize = str(self.cfg.payload_livemedia) + " MB"

        self.log.debug(_("Total size of all packages: %s") % (pkgsize), level=4)
        self.log.debug(_("Total size of all packages, installed: %s") % (installsize), level=4)

        pbar.destroy()

    def report_sizes(self):
        """Of all packages in the transaction report the installedsize and packagesize, sorted"""

        if not self.cfg.report_sizes:
            return

        list_installsize = []
        list_packagesize = []
        pkgs = self.cfg.yumobj.tsInfo.getMembers()
        for pkg in pkgs:
            if hasattr(pkg.po,"installedsize"):
                list_installsize.append("%d %s-%s-%s.%s" % (pkg.po.installedsize,pkg.po.name,pkg.po.version,pkg.po.release,pkg.po.arch))
            else:
                list_installsize.append("unknown %s-%s-%s.%s" % (pkg.po.name,pkg.po.version,pkg.po.release,pkg.po.arch))
            if hasattr(pkg.po,"packagesize"):
                list_packagesize.append("%d %s-%s-%s.%s" % (pkg.po.packagesize,pkg.po.name,pkg.po.version,pkg.po.release,pkg.po.arch))
            else:
                list_packagesize.append("unknown %s-%s-%s.%s" % (pkg.po.name,pkg.po.version,pkg.po.release,pkg.po.arch))

        list_installsize = self.numeric_rev_sort(list_installsize)
        list_packagesize = self.numeric_rev_sort(list_packagesize)

        print "==" + _("Report of the %d most space consuming packages") + "=="

        print "\nInstall Sizes:\n"
        i = 0
        for line in list_installsize:
            (size,name) = line.split(' ')
            (size,unit) = self.size_me(size)
            print "%s %s for %s" % (size,unit,name)
            i += 1
            if i > self.cfg.report_sizes_length:
                break

        print "\nPackage (RPM) Sizes:\n"
        i = 0
        for line in list_packagesize:
            (size,name) = line.split(' ')
            (size,unit) = self.size_me(size)
            print "%s %s for %s" % (size,unit,name)
            i += 1
            if i > self.cfg.report_sizes_length:
                break

    def size_me(self, val):
        ret_val = int(val)
        units = [ "B", "kB", "MB", "GB", "TB" ]
        while ret_val > 1024:
            units.pop(0)
            ret_val = ret_val / 1024

        return (ret_val,units[0])

    def numeric_rev_sort(self, alist):
        indices = map(self._generate_index, alist)
        decorated = zip(indices, alist)
        decorated.sort()
        decorated.reverse()
        return [ item for index, item in decorated ]

    def _generate_index(self, str):
        """
        Splits a string into alpha and numeric elements, which
        is used as an index for sorting"
        """
        #
        # the index is built progressively
        # using the _append function
        #
        index = []
        def _append(fragment, alist=index):
            if fragment.isdigit(): fragment = int(fragment)
            alist.append(fragment)

        # initialize loop
        prev_isdigit = str[0].isdigit()
        current_fragment = ''
        # group a string into digit and non-digit parts
        for char in str:
            curr_isdigit = char.isdigit()
            if curr_isdigit == prev_isdigit:
                current_fragment += char
            else:
                _append(current_fragment)
                current_fragment = char
                prev_isdigit = curr_isdigit
        _append(current_fragment)
        return tuple(index)

    def buildInstallationMedia(self):
        from revisor import pungi

        self.plugins.exec_hook("pre_compose_media_installation")

        # Actually do work.
        if not self.cfg.architecture == 'source':

            # init some
            mypungi = pungi.RevisorPungi(self)

            try:
                os.makedirs(os.path.join(mypungi.destdir,
                                         self.cfg.version,
                                         self.cfg.model,
                                         self.cfg.architecture))
            except:
                pass

            pkgdir = os.path.join(  mypungi.destdir,
                                    self.cfg.version,
                                    self.cfg.model,
                                    self.cfg.architecture,
                                    "os",
                                    self.cfg.product_path)

            if not os.path.exists(pkgdir):
                self.log.debug(_("Creating pkgdir: %s" % (pkgdir)))
                os.makedirs(pkgdir)
            else:
                shutil.rmtree(pkgdir, ignore_errors=True)
                os.makedirs(pkgdir)

            if self.cfg.getsource:
                pkgdir_src = os.path.join(  mypungi.destdir,
                                            self.cfg.version,
                                            self.cfg.model,
                                            "source",
                                            "SRPMS")

                if not os.path.exists(pkgdir_src):
                    self.log.debug(_("Creating pkgdir: %s" % (pkgdir_src)))
                    os.makedirs(pkgdir_src)
                else:
                    shutil.rmtree(pkgdir_src, ignore_errors=True)
                    os.makedirs(pkgdir_src)

            # Now for each txmbr in

            # Link the localPkg() result into the build tree
            pbar = self.progress_bar(_("Linking in packages"))
            failedHardLink = False
            total = float(len(self.polist))
            self.cfg.number_of_packages = total
            i = 0
            pos = self.polist
            if self.cfg.getsource:
                pos += self.srpmpolist
                total += float(len(self.srpmpolist))

            for po in pos:
                if po.arch == "src":
                    link_dir = pkgdir_src
                else:
                    link_dir = pkgdir

                try:
                    os.link(po.localPkg(), link_dir + "/" + os.path.basename(po.localPkg()))
                except OSError, e:
                    self.log.debug(_("Package hard link failed: %s: %s") % (e, link_dir + "/" + os.path.basename(po.localPkg())))
                    if e.errno == 17:
                        continue
                    elif e.errno == 18:
                        if self.cfg.copy_local:
                            self.log.debug(_("Copying: %s to %s") % (po.localPkg(), link_dir + "/" + os.path.basename(po.localPkg())))
                            shutil.copy2(po.localPkg(), link_dir + "/" + os.path.basename(po.localPkg()))
                        else:
                            self.log.debug(_("Symlinking: %s to %s") % (po.localPkg(), link_dir + "/" + os.path.basename(po.localPkg())))
                            try:
                                os.symlink(po.localPkg(), link_dir + "/" + os.path.basename(po.localPkg()))
                                self.log.debug(_("Package symlink succeeded"))
                            except OSError, e:
                                self.log.debug(_("Package link failed, trying copy: %s: %s") % (e, os.path.basename(po.localPkg())))
                                try:
                                    shutil.copy2(po.localPkg(), link_dir + "/" + os.path.basename(po.localPkg()))
                                except:
                                    pass
                    else:
                        try:
                            self.log.debug(_("Package link failed, trying copy: %s: %s") % (e, os.path.basename(po.localPkg())))
                            shutil.copy2(po.localPkg(), link_dir + "/" + os.path.basename(po.localPkg()))
                        except:
                            pass

                i += 1
                pbar.set_fraction(i/total)
            pbar.destroy()

            pbar = self.progress_bar(_("Creating Repository Information"), callback=True)
            pungicallback = revisor.progress.PungiCallback(pbar, pungi=mypungi, cfg=self.cfg)

            database = True
            if os.access("/etc/centos-release", os.R_OK):
                database = False
            elif os.access("/etc/redhat-release", os.R_OK) and not os.access("/etc/fedora-release", os.R_OK):
                database = False

            mypungi.doCreateRepo(database=database,callback=pungicallback)
            pbar.destroy()

            self.plugins.exec_hook('pre_exec_buildinstall')

            # FIXME: Optionally recompose installer images
            pbar = self.progress_bar(_("Building Installation Images"))

            if not os.access(os.path.join(self.cfg.working_directory, "revisor-pungi", self.cfg.version, self.cfg.model, self.cfg.architecture, "os", "isolinux", "initrd.img"), os.R_OK):
                pungicallback = revisor.progress.PungiCallback(pbar, pungi=mypungi, cfg=self.cfg)
                mypungi.doBuildinstall(callback=pungicallback)
            else:
                self.log.debug(_("Using already existing installer images"), level=4)

            pbar.destroy()

            # This doesn't work, doh!
#            # Nuke stage2.img
#            if self.cfg.install_nogr:
#                try:
#                    os.unlink(os.path.join(self.cfg.working_directory, "revisor-pungi", self.cfg.version, self.cfg.model, self.cfg.architecture, "os", "images", "stage2.img"))
#                except:
#                    pass

            self.plugins.exec_hook('post_exec_buildinstall')

            pbar = self.progress_bar(_("Linking in release notes"))
            pungicallback = revisor.progress.PungiCallback(pbar, pungi=mypungi, cfg=self.cfg)
            mypungi.doGetRelnotes(callback=pungicallback)
            pbar.destroy()

            tree_location = '%s/os' % os.path.join(mypungi.destdir,
                                                                    self.cfg.version,
                                                                    self.cfg.model,
                                                                    self.cfg.architecture)

            # Add updates.img into the tree
            if self.cfg.updates_img:
                self.log.debug(_("Copying updates.img from %s to %s") % (self.cfg.updates_img, os.path.join(tree_location, "images/updates.img")), level=8)
                shutil.copy(revisor.misc.get_file(self.cfg.updates_img), os.path.join(tree_location, 'images/updates.img'))

            # FIXME: Add linux updates to isolinux/isolinux.cfg as well

            if not self.cfg.copy_dir == "":
                if os.access(self.cfg.copy_dir, os.R_OK):
                    mypungi.doCopyDir(copy_dir=self.cfg.copy_dir)
                else:
                    self.log.warning(_("copy_dir '%s' not accessible") % self.cfg.copy_dir)

# Relevant code to USB Installation Media
# Has the size of the tree compared to the size of the USB Thumb Drive

            calc_size_str = subprocess.Popen(['/usr/bin/du',
                                            '-sbLl',
                                            tree_location],
                                stdout=subprocess.PIPE, stderr=subprocess.STDOUT).communicate()[0]
            calc_size_str = calc_size_str.split()[0]
            self.cfg.mediatypes["unified"]["size"] = int(calc_size_str) + 1
            self.log.debug(_("Size of the installation tree is %s MB") % str(int(calc_size_str) / 1024 / 1024))

            override_retval = ""
            # So, what would the number of discs be?
            for num in self.cfg.mediatypes["index"].keys():
                mediatype = self.cfg.mediatypes["index"][num]

                # Do we even compose this type?
                exec("retval = %s" % self.cfg.mediatypes[mediatype]["compose"])

                # If not, we want to just continue on to the next media
                if not retval:
                    self.cfg.mediatypes[mediatype]["compose"] = False
                    continue

                # Apparently we do compose this media type, but has a previous media
                # type been set to compose, and does it have multiple discs?
                # Because if it hasn't, there's no need to compose a DVD of whatever
                # fits on a single CD, for example.
                if not override_retval == "hihi":
                    self.cfg.mediatypes[mediatype]["compose"] = retval
                else:
                    self.cfg.mediatypes[mediatype]["compose"] = False

                if mediatype == "unified":
                    self.cfg.mediatypes[mediatype]["discs"] = 1
                else:
                    self.cfg.mediatypes[mediatype]["discs"] = int(math.ceil(self.cfg.mediatypes["unified"]["size"] / self.cfg.mediatypes[mediatype]["size"])) + 1

                if self.cfg.mediatypes[mediatype]["discs"] > 1:
                    self.cfg.do_packageorder = True
                else:
                    # Heej if my discnumber is one the rest doesn't have to compose!
                    override_retval = "hihi"

            # Now, we know everything!
            # And we lived happily ever after
            if self.cfg.gui_mode:
                self.gui.BuildMedia.extend_task_list()

# FIXME
# This runs without progress or anything
            if self.cfg.do_packageorder and self.cfg.pkgorder_file == "":
                import pkgorder

                # On EL5, the yum cache is placed under the installroot
                # That makes our lives difficult, hihi
                # So, make sure that if we're on el_linux, the repository gets
                # bind mounted and configured appropriately

                pbar = self.progress_bar(_("Running pkgorder"))

                ds = pkgorder.PackageOrderer(self)

                try:
                    os.makedirs(os.path.join(self.cfg.yumobj.cachedir,"anaconda","headers"))
                except:
                    pass

                ds.createConfig()

                ds.setup()

                pbar.set_fraction(1.0/28.0)

                #ds.printMatchingPkgs("kernel*")
                #ds.printMatchingPkgs("iscsi-*")
                #ds.printMatchingPkgs("*firmware*")

                ds.addPackages([ "kernel*", "iscsi-*", "*firmware*" ])

                pbar.set_fraction(2.0/38.0)

                if self.cfg.version == 7:
                    # Here is where we add groups
                    ds.addGroups(["core", "base", "text-internet"])

                    pbar.set_fraction(3.0/38.0)

                    ds.addGroups(["base-x", "dial-up",
                                    "graphical-internet", "editors",
                                    "graphics", "gnome-desktop", "sound-and-video",
                                    "printing"])

                    pbar.set_fraction(11.0/38.0)

                    ds.addGroups(["office", "engineering-and-scientific",
                                "authoring-and-publishing", "games"])

                    pbar.set_fraction(15.0/38.0)

                    ds.addGroups(["web-server", "ftp-server", "sql-server",
                                "mysql", "server-cfg", "dns-server",
                                "smb-server", "admin-tools"])

                    pbar.set_fraction(23.0/38.0)

                    ds.addGroups(["kde-desktop", "development-tools", "development-libs",
                                "gnome-software-development", "eclipse",
                                "x-software-development",
                                "java-development", "kde-software-development",
                                "mail-server", "network-server", "legacy-network-server"])
                else:
                    # These are default
                    ds.addGroups(["core", "base", "base-x"])

                    pbar.set_fraction(3.0/38.0)

                    ds.addGroups(["dial-up", "text-internet", "editors", "fonts", "games"])

                    pbar.set_fraction(8.0/38.0)

                    ds.addGroups(["gnome-desktop", "graphical-internet"])

                    pbar.set_fraction(10.0/38.0)

                    ds.addGroups(["hardware-support", "java", "office", "graphics", "printing"])

                    pbar.set_fraction(15.0/38.0)

                    ds.addGroups(["legacy-fonts", "sound-and-video"])

                    pbar.set_fraction(17.0/38.0)

                    ds.addGroups(["engineering-and-scientific",
                                "authoring-and-publishing",
                                "kde-desktop"])

                    pbar.set_fraction(20.0/38.0)

                    ds.addGroups(["web-server", "ftp-server", "sql-server",
                                "mysql", "server-cfg", "dns-server",
                                "smb-server", "admin-tools"])

                    pbar.set_fraction(28.0/38.0)

                    ds.addGroups(["development-tools", "development-libs",
                                "gnome-software-development", "eclipse",
                                "x-software-development",
                                "java-development", "kde-software-development",
                                "mail-server", "network-server", "legacy-network-server"])

                pbar.set_fraction(34.0/38.0)

                ds.addGroups(["news-server", "legacy-software-development"])

                # Everthing else but kernels
                for po in ds.pkgSack.returnPackages():
                    fpattern = "%s*" % po.name
                    ds.printMatchingPkgs(fpattern)

                pbar.destroy()

#                if el_linux:
#                    rc = subprocess.call(["/bin/umount", dst])

            elif self.cfg.do_packageorder and os.access(self.cfg.pkgorder_file, os.R_OK):
                self.log.debug(_("Not running package ordering, using file %s instead") % self.cfg.pkgorder_file, level=9)
            else:
                self.log.debug(_("Not running package ordering"), level=9)

            if not self.cfg.kickstart_save == "":
                # FIXME: Maybe we want to offer in destdir dumping along with absolute dumping?
                # For now, it's just "give us a location to save to"
                f = open(self.cfg.kickstart_save, "w")
                f.write(self.cfg.ksobj.__str__())
                f.close()

            if self.cfg.kickstart_include:
# FIXME
                # This needs some more work because the kickstart file may have been
                # generated during our run, partially modified during our run, and may not have
                # been used at all
                if self.cfg.use_kickstart_file:
                    shutil.copyfile(self.cfg.kickstart_file, os.path.join(mypungi.topdir,"ks.cfg"))
                else:
                    f = open(os.path.join(mypungi.topdir,"ks.cfg"), "w")
                    f.write(self.cfg.ksobj.__str__())
                    f.close()

                if self.cfg.kickstart_default:
                    f = open(os.path.join(mypungi.topdir,"isolinux","isolinux.cfg"),"rw+")
                    buf = f.read()
                    # Replace an existent "label ks" because we want to append our own
                    buf = buf.replace("label ks\n  kernel vmlinuz\n  append ks initrd=initrd.img\n","")
#                    buf = buf.replace("#prompt 1","prompt 1")
                    buf = buf.replace("menu default\n","")
                    buf = buf.replace("default linux","default ks")
#                    buf = buf.replace("default vesamenu.c32","default ks")
                    f.seek(0)
                    f.write(buf)
                    f.write("label ks\n  menu label " + _("^Install using kickstart") + "\n  menu default\n  kernel vmlinuz\n  append initrd=initrd.img ks=cdrom:/ks.cfg %s\n" % self.cfg.im_append)
                    f.close()

# Do some really neat routine here
            for num in self.cfg.mediatypes["index"].keys():
                mediatype = self.cfg.mediatypes["index"][num]

                discs = self.cfg.mediatypes[mediatype]["discs"]
                size = self.cfg.mediatypes[mediatype]["size"]
                discdir = self.cfg.mediatypes[mediatype]["discdir"]
                label = self.cfg.mediatypes[mediatype]["label"]

                if self.cfg.mediatypes[mediatype]["compose"]:
                    os.symlink(mypungi.topdir,"%s-%s" % (mypungi.topdir, discdir))

                    for i in range(2, discs+1):
                        try:
                            os.makedirs("%s-%s-disc%d" % (mypungi.topdir, discdir, i))
                        except:
                            pass

                    if discs > 1:
                        # Split Tree
                        pbar = self.progress_bar(_("Splitting Build Tree (%s)") % label)
                        pungicallback = revisor.progress.PungiCallback(pbar, pungi=mypungi, cfg=self.cfg)
                        mypungi.doSplittree(media_size=size,discdir=discdir,callback=pungicallback)
                        pbar.destroy()

                        # Split repo
                        pbar = self.progress_bar(_("Splitting Repository (%s)") % label)
                        pungicallback = revisor.progress.PungiCallback(pbar, pungi=mypungi, cfg=self.cfg)
                        mypungi.doCreateSplitrepo(discdir=discdir,callback=pungicallback)
                        pbar.destroy()

                    # Now that we're done splitting the tree, re-init some numbers (cause the number of discs may have changed)
                    discs = self.cfg.mediatypes[mediatype]["discs"]

# Not sure this fixes anything
#                    os.symlink(os.path.join(mypungi.topdir,"repodata"), os.path.join(mypungi.topdir,self.cfg.product_path,"repodata"))

                    # Create ISOs
                    if discs > 1:
                        for disc in range(1, discs+1):
                            pbar = self.progress_bar(_("Creating %s ISO Image #%d") % (label, disc))
                            pungicallback = revisor.progress.PungiCallback(pbar, pungi=mypungi, cfg=self.cfg)
                            mypungi.doCreateIso(mediatype=mediatype, disc=disc, callback=pungicallback)
                            pbar.destroy()
                    else:
                        pbar = self.progress_bar(_("Creating %s ISO Image") % label)
                        pungicallback = revisor.progress.PungiCallback(pbar, pungi=mypungi, cfg=self.cfg)
                        mypungi.ALL_workaround()
                        mypungi.doCreateIso(mediatype=mediatype, callback=pungicallback)
                        pbar.destroy()

# Insert code to USB Installation Media
# Compose the USB Installation Media here

            if len(self.cfg.built_iso_images) > 0:

                if not self.cfg.skip_implantisomd5:
                    # For all images but source images; implant the md5 into the ISO for the media check
                    # FIXME: Well, it seems we don't deal with source images here.
                    pbar = self.progress_bar(_("Implanting MD5 into ISO Images"))
                    total = float(len(self.cfg.built_iso_images))
                    cur = 0.0
                    for built_image in self.cfg.built_iso_images:
                        isofile = built_image["location"]
                        pbar.set_markup(isofile)
                        #mediatype = built_image["mediatype"]
                        #if not mediatype == 'source':
                        self.log.debug(_("Implanting md5 into ISO Image: %s" % (isofile)))
                        if os.access("/usr/lib/anaconda-runtime/implantisomd5", os.R_OK):
                            self.run_command(['/usr/lib/anaconda-runtime/implantisomd5', isofile])
                        elif os.access("/usr/bin/implantisomd5", os.R_OK):
                            self.run_command(['/usr/bin/implantisomd5', isofile])
                        else:
                            self.log.error(_("Cannot implant ISO md5sum"), recoverable=True)

                        cur += 1.0
                        pbar.set_fraction(cur/total)
                    pbar.destroy()

                if not self.cfg.skip_sha1sum:
                    # Do some SHA1SUMMONING
                    pbar = self.progress_bar(_("Creating SHA1SUMs for Images"))
                    total = float(len(self.cfg.built_iso_images))
                    cur = 0.0
                    for built_image in self.cfg.built_iso_images:
                        isofile = os.path.basename(built_image["location"])
                        pbar.set_markup(isofile)
                        isodir = os.path.dirname(built_image["location"])
                        # shove the sha1sum into a file
                        sha1file = open(os.path.join(isodir,"SHA1SUM"), 'a')
                        self.run_command(['/usr/bin/sha1sum', isofile], rundir=isodir, output=sha1file)
                        sha1file.close()
                        cur += 1.0
                        pbar.set_fraction(cur/total)
                    pbar.destroy()

        if self.cfg.media_utility_rescue:
            pbar = self.progress_bar(_("Creating Rescue ISO Image"))
            pungicallback = revisor.progress.PungiCallback(pbar, pungi=mypungi, cfg=self.cfg)
            mypungi.create_rescue_disk(callback=pungicallback)
            pbar.destroy()

        # Does not work yet because anaconda doesn't allow installations from expanded trees on a hard drive
        #if self.cfg.media_installation_usb:
            #pbar = self.progress_bar(_("Creating USB Key Installer"))
            #usb_file = os.path.join(self.cfg.working_directory,"%s-%s-%s-USB.img" % (self.cfg.iso_basename, self.cfg.version, self.cfg.architecture))
            #self.run_command(["qemu-img", "create", usb_file, self.cfg.usb_size])
            #self.run_command(["mkdosfs", usb_file])

            #usb_mount_location = os.path.join(self.cfg.working_directory, "revisor-usb")

            ## Maybe make the directory
            #try:
                #os.makedirs(usb_mount_location)
            #except:
                #pass

            #usb_mount = imgcreate.fs.LoopbackMount(usb_file, usb_mount_location)
            #usb_mount.mount()

            ## Copy files over
            ##shutil.copytree(mypungi.topdir, usb_mount_location)
            #src = mypungi.topdir
            #dst = usb_mount_location

            #names = os.listdir(src)
            #for name in names:
                #srcname = os.path.join(src, name)
                #dstname = os.path.join(dst, name)
                #if os.path.isdir(srcname):
                    #shutil.copytree(srcname, dstname)
                #else:
                    #shutil.copy2(srcname, dstname)
            ## Modify isolinux.cfg
            ## FIXME: Some action here
            ## Move it to syslinux.cfg
            #shutil.move(os.path.join(usb_mount_location,"isolinux","isolinux.cfg"),os.path.join(usb_mount_location,"isolinux","syslinux.cfg"))
            ## Move isolinux/ to syslinux/
            #shutil.move(os.path.join(usb_mount_location,"isolinux"),os.path.join(usb_mount_location,"syslinux"))
            ## Unmount disk
            #usb_mount.unmount()
            ## Run syslinux
            #self.run_command(["syslinux", "-d", "syslinux", usb_file])

        self.cfg.plugins.exec_hook('post_compose_media_installation')

        if self.cfg.getsource:
            tree_dst = os.path.join(self.cfg.destination_directory,"os","source")
            tree_src = os.path.join(mypungi.destdir,self.cfg.version,self.cfg.model,"source")
            self.log.debug(_("Copying %s to %s") % (tree_src,tree_dst), level=1)
            try:
                if self.cfg.copy_local:
                    shutil.copytree(tree_src,tree_dst)
                else:
                    try:
                        shutil.move(tree_src,tree_dst)
                    except Exception, e:
                        self.log.error(_("Moving of the source tree failed (trying copy):\n\n%s") % '\n'.join(e), recoverable=True)
                        shutil.copytree(tree_src,tree_dst)
            except Exception, e:
                self.log.error(_("Copying of the source tree failed:\n\n%s") % '\n'.join(e), recoverable=True)

        if self.cfg.media_installation_tree:
            tree_dst = os.path.join(self.cfg.destination_directory,"os",self.cfg.architecture)
            tree_src = mypungi.topdir
            self.log.debug(_("Copying %s to %s") % (tree_src,tree_dst), level=1)
            try:
                shutil.copytree(tree_src,tree_dst)
            except:
                self.log.error(_("Copying of the installation tree failed:\n\n%s") % '\n'.join(e), recoverable=True)

    def buildLiveMedia(self):
        # Get our object from pilgrim
        liveImage = revisor.image.LiveImageCreator(self)

        pbar = self.progress_bar(_("Creating ext3 filesystem"))
        liveImage.mount(self.cfg.lm_base_on, self.cfg.yumobj.conf.cachedir)
        pbar.destroy()

        liveImage.install()

        pbar = self.progress_bar(_("Configuring System"))
        liveImage.configure()

        if self.cfg.lm_user_configuration:
            useradd = ['useradd']
            if len(self.cfg.lm_user_comment) > 0:
                useradd.extend(['-c', self.cfg.lm_user_comment])
            useradd.append(self.cfg.lm_user_name)
            self.run_command(useradd, preexec_fn=liveImage._instroot)

            if self.cfg.lm_user_auto_login:
                f = open("%s%s" % (liveImage._instroot,"/etc/gdm/custom.conf"),"a")
                f.write(self.cfg.gdm_auto_login % {'username': self.cfg.lm_user_name})
                f.close()

            if self.cfg.lm_user_wheel:
                self.run_command(["gpasswd", "-a", self.cfg.lm_user_name, "wheel"], preexec_fn=liveImage._instroot)

                if self.cfg.lm_wheel_sudo_no_passwd:
                    self.run_command(["sed", "-i", "-e", "'s/^#\s*%wheel.*NOPASSWD/%wheel ALL=(ALL) NOPASSWD: ALL/g'", "/etc/sudoers"], preexec_fn=liveImage._instroot)

        pbar.destroy()

        if self.cfg.lm_chroot_shell:
            print "Launching shell. Exit to continue."
            print "----------------------------------"
            liveImage.launch_shell()
        liveImage.unmount()
        liveImage.package(destdir=os.path.join(self.cfg.destination_directory,"live"))

    def set_run_command_callback(self, callback=None):
        self.run_command_callback = callback

    def run_command(self, command, rundir=None, output=subprocess.PIPE, error=subprocess.STDOUT, inshell=False, env=None, callback=None, preexec_fn=None):
        """Run a command and log the output."""

        if rundir == None:
            rundir = os.path.join(self.cfg.working_directory,"revisor-rundir")
        self.log.debug(_("Setting rundir to %s") % rundir)

        if not os.access(rundir, os.R_OK):
            try:
                os.makedirs(rundir)
            except:
                self.log.debug(_("Directory %s could not be created. Aborting"), recoverable=False)

        self.log.debug(_("Running command: %s") % ' '.join(command))
        self.log.debug(_("Extra information: %s %s %s") % (rundir, inshell, env))

        if output == subprocess.PIPE:
            p1 = subprocess.Popen(command, cwd=rundir, stdout=output, stderr=error, shell=inshell, env=env, preexec_fn=preexec_fn)

            while p1.poll() == None:
                try:
                    returncode = int(p1.poll())
                except:
                    if output == subprocess.PIPE:
                        try:
                            line = p1.stdout.readline()
                            if not line == "":
                                callback.parse_line(command, line)
                        except:
                            pass
                    if error == subprocess.PIPE:
                        try:
                            line = p1.stderr.readline()
                            if not line == "":
                                self.log.debug(line.strip())
                        except:
                            pass
        else:
            p1 = subprocess.Popen(command, cwd=rundir, stdout=output, stderr=error, preexec_fn=preexec_fn)
            (out, err) = p1.communicate()

        if p1.returncode != 0:
            self.log.error(_("Got an error from %s (return code %s)") % (command[0],p1.returncode), recoverable=self.cfg.gui_mode)
