#
# Copyright 2007 Fedora Unity
#
# Jonathan Steffan <jon a fedoraunity.org>
# Jeroen van Meeuwen <kanarip a fedoraunity.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import logging
import os
import revisor
import sys
import urlgrabber
import urlparse
import yum

from rhpl.translate import _

global resolved_deps

def check_uid():
    """Check if the user has enough permissions to perform this task"""
    if os.geteuid() != 0:
        print >> sys.stderr, _("This tool has to run with root privileges. Aborting")
        sys.exit(1)

def check_selinux(log=None):
    """Check if the composing host has SELinux in enforcing (which will cause the build to fail)"""
    if os.path.exists("/selinux/enforce"):
        f = open("/selinux/enforce")
        buf = f.read().strip()
        f.close()
        if buf == "1":
            # SELinux in enforcing mode
            log.error(_("SELinux is in enforcing mode on this host. Composing media will fail. Please set SELinux to permissive mode."), recoverable=False)
        else:
            # SELinux in permissive mode
            pass
    else:
        log.warning(_("SELinux on this host is disabled. Composed media will not have SELinux, and as a result the system you install from the composed media will not have SELinux either."))

def get_file(url, working_directory="/var/tmp"):
    """Gets a file from url and returns the file name (full path)"""

    # Check if it isn't a file already
    if os.path.isfile(url): return url

    # Otherwise it might be an actual url
    file_basename = os.path.basename(urlparse.urlparse(url).path)
    file_name = os.path.join(working_directory, file_basename)

    if not check_file(file_name, destroy=True):
        # File does not exist or wasn't valid. Download the file.
        download_file(url, file_name)
    else:
        # There should be no else to this one
        pass

    return file_name

def download_file(url, file_name, title=None):
    urlgrabber.urlgrab(url, file_name, copy_local=1)

def check_file(file_name, checksum=None, destroy=False):
    """
    Checks if a file exists. Basically returns True if the file exists, unless the
    checksum doesn't check out or destroy has been set to True.

    If the file exists:
        Checksum:
            If specified, run the checksum and return:
                - True if the checksum checks out.
                - False if it doesn't and destroy the file.
            If not specified:
                - Continue
        Destroy:
            If True:
                - Destroy the file and return False
            else:
                - Return True
    else:
        - return False

    """
    if os.access(file_name, os.R_OK):
        if not checksum == None:
            if file_checksum(file_name, checksum):
                return True
            else:
                return False
        elif destroy:
            os.remove(file_name)
            return False
        else:
            return True
    else:
        return False

def resolve_pkgs(yumobj, package_list, log=None):
    """Given a list of 'packages', resolves these 'packages' into package names so that later
    they can be selected. A good example is the suggested package 'system-logos', which just so
    happens to be 'fedora-logos', or 'generic-logos', or 'redhat-logos', or 'centos-logos'. We don't
    know, and neither do you"""
    final_package_list = []

    for pkg in package_list:
        if hasattr(yumobj.pkgSack,"contains"):
            has_pkg = yumobj.pkgSack.contains(pkg)
        else:
            try:
                pkgs = yumobj.pkgSack.returnNewestByName(pkg)
                if len(pkgs) > 0:
                    has_pkg = True
                else:
                    has_pkg = False
            except yum.Errors.PackageSackError, e:
                has_pkg = False

        if has_pkg:
            final_package_list.append(pkg)
            if not log == None:
                log.debug(_("Resolved %s") % (pkg), level=9)
        else:
            # If the package isn't in, search the provides
            pkgs = yumobj.whatProvides(pkg, None, None).returnPackages()
            if len(pkgs) > 0:
                final_package_list.append(pkg)
                if not log == None:
                    log.debug(_("Resolved %s") % (pkg), level=9)
            else:
                if not log == None:
                    log.debug(_("Looking to resolve package %s to a Provides, but we still can't find it.") % pkg, level=2)

    return final_package_list

def resolve_dependencies_ugly(yumobj, logger=None, pbar=None):
    def get_package_deps(po, logger=None):
        """Add the dependencies for a given package to the
           transaction info"""

        if logger == None:
            logger = revisor.logger.Logger()

        logger.debug(_("Checking dependencies for %s.%s") % (po.name, po.arch), level=8)

        if not pbar == None:
            pbar.cur_task += 1.0

        reqs = po.requires
        provs = po.provides

        for req in reqs:
            if resolved_deps.has_key(req):
                if not pbar == None:
                    pbar.cur_task += 1.0
                    pbar.set_fraction(pbar.cur_task/pbar.num_tasks)
                continue

            (r,f,v) = req
            if r.startswith('rpmlib(') or r.startswith('config('):
                if not pbar == None:
                    pbar.cur_task += 1.0
                    pbar.set_fraction(pbar.cur_task/pbar.num_tasks)
                continue

            deps = yumobj.whatProvides(r, f, v).returnPackages()

            if not deps:
                if not pbar == None:
                    pbar.cur_task += 1.0
                logger.warning(_("Unresolvable dependency %s %s %s in %s.%s") % (r, f, v, po.name, po.arch))
                continue

            depsack = yum.packageSack.ListPackageSack(deps)

            for dep in depsack.returnNewestByNameArch():
                yumobj.tsInfo.addInstall(dep)
                logger.debug(_("Added %s-%s:%s-%s.%s for %s-%s:%s-%s.%s (requiring %s %s %s)") % (dep.name, dep.epoch, dep.version, dep.release, dep.arch, po.name, po.epoch, po.version, po.release, po.arch, r, f, v), level=9)

    if logger == None:
        logger = revisor.logger.Logger()

    logger.debug(_("Checking dependencies - allowing conflicts within the package set"), level=5)

    reqs = []
    for txmbr in yumobj.tsInfo.getMembers():
        reqs.append(txmbr.po.requires)
    reqs = yum.misc.unique(reqs)

    resolved_deps = {}
    final_pkgobjs = {}

    moretoprocess = True
    while moretoprocess: # Our fun loop
        moretoprocess = False
        for txmbr in yumobj.tsInfo.getMembers():
            if not final_pkgobjs.has_key(txmbr.po):
                final_pkgobjs[txmbr.po] = None
                get_package_deps(txmbr.po, logger)
                moretoprocess = True

    #yumobj.tsInfo.makelists()

def get_repourls(yumobj):
    repositories = []
    for repo in yumobj.repos.listEnabled():
        if hasattr(repo, "baseurl"):
            if isinstance(repo.baseurl, list):
                repositories.append(repo.baseurl[0])
            else:
                repositories.append(repo.baseurl)
        #if hasattr(repo,"mirrorlist"):
            #print "Using mirrorlist %s from repo %s" % (repo.mirrorlist,repo.id)
            #return repo.mirrorlist
    return repositories
