#
# Copyright 2007 Fedora Unity
#
# Jonathan Steffan <jon a fedoraunity.org>
# Jeroen van Meeuwen <kanarip a fedoraunity.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Based on 'anaconda-runtime' pkgorder by:
# Paul Nasrat <pnasrat@redhat.com>
# Copyright 2005 Red Hat, Inc.
#

import os
import glob
import revisor.misc
import rpm
import rpmUtils
import shutil
import string
import sys
import yum
from yum.packageSack import PackageSack

import tempfile

#import iutil
#

# Translation
from rhpl.translate import _

class PackageOrderer(yum.YumBase):
    """The revisor package orderer that allows changes to yum and RPM in between releases,
    as well as package ordering customization"""

    def __init__(self, base):
        self.base = base
        self.cfg = base.cfg
        self.log = base.log

        yum.YumBase.__init__(self)
        self.arch = self.cfg.architecture

        self.processed = {}

        # Boo
        if self.arch == "i386":
            self.arch = "i686"

        # We're gonna use this an awful lot
        self.toppath = os.path.join(self.cfg.working_directory,"revisor-pungi",self.cfg.version,self.cfg.model,self.cfg.architecture,"os")

        self.cfg.pkgorder_file = os.path.join(self.cfg.working_directory,"revisor-pungi","pkgorder-file")
        # If we have a pkgorder file set already, see if we can use that one.
        self.log.debug("Setting pkgorder_file to %s" % self.cfg.pkgorder_file)

        self.testpath = tempfile.mkdtemp(dir=os.path.join(self.cfg.working_directory,"revisor-pungi"), prefix="pkgorder-root-")
        os.makedirs(os.path.join(self.testpath,"etc"))

    def addGroups(self, groupLst):
        self.initActionTs()
        map(self.selectGroup, filter(lambda x: self.comps.has_group(x), groupLst))

        if self.cfg.pkgorder_style == "yum":
            self.resolveDeps()
        else:
            revisor.misc.resolve_dependencies_ugly(self.cfg.yumobj, self.cfg.log)

        self.processTransaction()

    def addPackages(self, packageLst):
        self.initActionTs()

        (exactmatched, matched, unmatched) = yum.packages.parsePackages(self.pkgSack.returnPackages(), packageLst, casematch=0)
        matches = exactmatched + matched

        map(self.install, filter(lambda x: self.pkgSack.returnNewestByName(), matches))

        if self.cfg.pkgorder_style == "yum":
            self.resolveDeps()
        else:
            revisor.misc.resolve_dependencies_ugly(self.cfg.yumobj, self.cfg.log)

        self.processTransaction()

    def processTransaction(self):
        del self.ts
        self.initActionTs()
        self.populateTs(keepold=0)
        self.ts.check()
        self.ts.order()
        for (hdr, path) in self.ts.ts.getKeys():
            fname = os.path.basename(path)
            fpattern = "%s*" % fname.rsplit('.', 2)[0]
            self.printMatchingPkgs(fpattern)

    def createConfig(self):
        yumconfstr = """
[main]
distroverpkg=redhat-release
gpgcheck=0
reposdir=/dev/null
exclude=*debuginfo*
debuglevel=9
logfile=/var/log/yum.log
cachedir=%(cachedir)s

[anaconda]
name=Anaconda
baseurl=file://%(toppath)s
enabled=1
    """ % { "toppath": self.toppath, "cachedir": self.cfg.yumobj.conf.cachedir }

        try:
            f = open(os.path.join(self.testpath,"etc","yum.conf"),"w")
            f.write(yumconfstr)
            f.close()
            return os.path.join(self.testpath,"etc","yum.conf")
        except:
            self.log.error(_("Unable to create yum configuration file for package ordering at %s") % os.path.join(self.testpath,"etc","yum.conf"))

    def printMatchingPkgs(self, fpattern):
        if os.path.isdir("%s/%s/RPMS" % (self.toppath, self.cfg.product_path)):
            matches = glob.glob("%s/%s/RPMS/%s" % (self.toppath, self.cfg.product_path, fpattern))
        elif os.path.isdir("%s/%s" %(self.toppath, self.cfg.product_path)):
            matches = glob.glob("%s/%s/%s" % (self.toppath, self.cfg.product_path, fpattern))
        # F8 prep
        elif os.path.isdir("%s/Packages" % (self.toppath)):
            matches = glob.glob("%s/Packages/%s" % (self.toppath, fpattern))
        else:
            matches = glob.glob("%s/%s" % (toppath, fpattern))

        f = open(self.cfg.pkgorder_file,"a")
        for match in matches:
            mname = os.path.basename(match)
            if self.processed.has_key(mname): continue
            self.processed[mname] = True
            f.write(mname + "\n")
            self.log.debug(_("-> package %s") % mname)
        f.close()

    def doFileLogSetup(self, uid, logfile):
        pass

    def doLoggingSetup(self, debuglevel, errorlevel):
        pass

    def setup(self,pbar=None):

        if not pbar == None:
            self.pbar = pbar

        if hasattr(self,"_getConfig"):
            self._getConfig(fn=self.createConfig(), root=self.testpath, init_plugins=False)
        else:
            self.doConfigSetup(fn=self.createConfig(), root=self.testpath, init_plugins=False)
            self.log.debug(_("Using deprecated YUM function: %s()") % "doConfigSetup")
        self.doRepoSetup()
        if hasattr(self,"_getTs"):
            self._getTs()
        else:
            self.log.debug(_("Using deprecated YUM function: %s()") % "doTsSetup")
            self.doTsSetup()

        if hasattr(self,"_getGroups"):
            self._getGroups()
        elif hasattr(self,"doGroupSetup"):
            self.doGroupSetup()

        if hasattr(self,"_getRpmDB"):
            self._getRpmDB()
        elif hasattr(self,"doSetupRPMDB"):
            self.log.debug(_("Using deprecated YUM function: %s()") % "doSetupRPMDB")
            self.doSetupRPMDB()
        elif hasattr(self,"doRpmDBSetup"):
            self.log.debug(_("Using deprecated YUM function: %s()") % "doRpmDBSetup")
            self.doRpmDBSetup()

        if not pbar == None:
            self.repos.callback = pbar

        if hasattr(self,"_getRepos"):
            self._getRepos()
        else:
            self.log.debug(_("Using deprecated YUM function: %s()") % "doRepoSetup")
            self.doRepoSetup()

#            self.log.debug(_("Arch list = %s") % self.cfg.arch_list)

        try:
            if hasattr(self,"_getSacks"):
                self._getSacks(archlist=self.cfg.arch_list)
            else:
                self.log.debug(_("Using deprecated YUM function: %s()") % "doSackSetup")
                self.doSackSetup(archlist=self.cfg.arch_list)
        except:
            pass

        if not pbar == None:
            self.pbar.destroy()

    def getDownloadPkgs(self):
        pass
